

# ===========================================================================================0
# TESTTEXTE
#          

# Preprocess
TESTTEXT_0 = '''Hällö, „Welt“! <i>Hel 9lo</i> <b>Wo9 rld</b>! Das Wetter ist gut. Wie könnte`s not? A Öder nicht? Nic nemůže letět Rychlostí vyšší, než 300 tisíc kilometrů za sekundu! Hel 9lo Wo9rld! Th3     weather_is really g00d today, isn't it? Das geht    super, nöß?   Riß_2'''


# Kokosnuss
TESTTEXT_DE1 = '''»Dies ist Björns Buch.«
In einer Bucht unterhalb der schönsten Höhle gehen sie vor Anker.
Nachdem sie Feuerholz gesammelt haben, angeln sie für das Abendessen ein paar Fische.
»Ziemlich kalt hier«
, sagte Matilda und fröstelt.
»Dort hinten«
, sagt Oskar,
»habe ich vorhin ein Eisbärfell gesehen.
Wenn du das trocknest, hast du eine schöne warme Decke.«
Kokosnuss stutzt.
»Ein Eisbärfell?
Das muss ich sehen!'''  



TESTTEXT_EN1 = '''Apple is looking at buying U.K. startup for $1 billion.
I was reading the paper.'''




