
#print('bj_nlp init')


   
# ==================================================================================================
# bj_nlp Files
# ==================================================================================================

from bj_nlp.Testtext         import *
from bj_nlp.Preprocessing    import *
from bj_nlp.blpClass         import *
from bj_nlp.translate_tags   import *
from bj_nlp._PandaziseMixin  import *
from bj_nlp._UtilsMixin      import *


# Custom Components. 
# Alle Custom Components sind in blpClass registriert.
from bj_nlp.c01_merger       import *
from bj_nlp.c05_wrede        import *
from bj_nlp.c10_wortschatz   import *








